import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:unicons/unicons.dart';

import 'package:scrobbles/config/default_settings.dart';
import 'package:scrobbles/cubit/settings_cubit.dart';
import 'package:scrobbles/ui/widgets/app_titles.dart';

class SettingsForm extends StatefulWidget {
  const SettingsForm({super.key});

  @override
  State<SettingsForm> createState() => _SettingsFormState();
}

class _SettingsFormState extends State<SettingsForm> {
  final usernameController = TextEditingController();
  final securityTokenController = TextEditingController();

  int discoveriesDaysCount = DefaultSettings.defaultDiscoveriesDaysCount;
  int distributionDaysCount = DefaultSettings.defaultDistributionDaysCount;
  int statisticsRecentDaysCount = DefaultSettings.defaultStatisticsRecentDaysCount;
  int timelineDaysCount = DefaultSettings.defaultTimelineDaysCount;
  int topArtistsDaysCount = DefaultSettings.defaultTopArtistsDaysCount;
  int newArtistsCount = DefaultSettings.defaultNewArtistsCount;
  int newTracksCount = DefaultSettings.defaultNewTracksCount;

  List<bool> _selectedDiscoveriesDaysCount = [];
  List<bool> _selectedDistributionDaysCount = [];
  List<bool> _selectedStatisticsRecentDaysCount = [];
  List<bool> _selectedTimelineDaysCount = [];
  List<bool> _selectedTopArtistsDaysCount = [];
  List<bool> _selectedNewArtistsCount = [];
  List<bool> _selectedNewTracksCount = [];

  @override
  void didChangeDependencies() {
    SettingsCubit settings = BlocProvider.of<SettingsCubit>(context);

    usernameController.text = settings.getUsername();
    securityTokenController.text = settings.getSecurityToken();

    discoveriesDaysCount = settings.getDiscoveriesDaysCount();
    distributionDaysCount = settings.getDistributionDaysCount();
    statisticsRecentDaysCount = settings.getStatisticsRecentDaysCount();
    timelineDaysCount = settings.getTimelineDaysCount();
    topArtistsDaysCount = settings.getTopArtistsDaysCount();
    newArtistsCount = settings.getNewArtistsCount();
    newTracksCount = settings.getNewTracksCount();

    _selectedDiscoveriesDaysCount = DefaultSettings.allowedDaysCountValues
        .map((e) => (e == discoveriesDaysCount))
        .toList();
    _selectedDistributionDaysCount = DefaultSettings.allowedDaysCountValues
        .map((e) => (e == distributionDaysCount))
        .toList();
    _selectedStatisticsRecentDaysCount = DefaultSettings.allowedDaysCountValues
        .map((e) => (e == statisticsRecentDaysCount))
        .toList();
    _selectedTimelineDaysCount =
        DefaultSettings.allowedDaysCountValues.map((e) => (e == timelineDaysCount)).toList();
    _selectedTopArtistsDaysCount =
        DefaultSettings.allowedDaysCountValues.map((e) => (e == topArtistsDaysCount)).toList();
    _selectedNewArtistsCount =
        DefaultSettings.allowedCountValues.map((e) => (e == newArtistsCount)).toList();
    _selectedNewTracksCount =
        DefaultSettings.allowedCountValues.map((e) => (e == newTracksCount)).toList();

    super.didChangeDependencies();
  }

  @override
  void dispose() {
    usernameController.dispose();
    securityTokenController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    void saveSettings() {
      BlocProvider.of<SettingsCubit>(context).setValues(
        username: usernameController.text,
        securityToken: securityTokenController.text,
        discoveriesDaysCount: discoveriesDaysCount,
        distributionDaysCount: distributionDaysCount,
        statisticsRecentDaysCount: statisticsRecentDaysCount,
        timelineDaysCount: timelineDaysCount,
        topArtistsDaysCount: topArtistsDaysCount,
        newArtistsCount: newArtistsCount,
        newTracksCount: newTracksCount,
      );
    }

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      mainAxisSize: MainAxisSize.max,
      children: <Widget>[
        SizedBox(height: 8),
        AppTitle2(text: tr('settings_title_global')),

        // Username
        Text('settings_label_username').tr(),
        TextFormField(
          controller: usernameController,
          decoration: InputDecoration(
            border: UnderlineInputBorder(),
            suffixIcon: ElevatedButton(
              style: ElevatedButton.styleFrom(
                shape: new RoundedRectangleBorder(
                  borderRadius: new BorderRadius.circular(6.0),
                ),
              ),
              child: Icon(UniconsLine.save),
              onPressed: () {
                saveSettings();
              },
            ),
          ),
        ),

        SizedBox(height: 8),
        AppTitle2(text: tr('settings_title_days_count')),

        // Statistics (recent)
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_statistics_recent_days_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  statisticsRecentDaysCount = DefaultSettings.allowedDaysCountValues[index];
                  for (int i = 0; i < _selectedStatisticsRecentDaysCount.length; i++) {
                    _selectedStatisticsRecentDaysCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedStatisticsRecentDaysCount,
              children: DefaultSettings.allowedDaysCountValues
                  .map((e) => Text(e.toString()))
                  .toList(),
            ),
          ],
        ),

        // Timeline
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_timeline_days_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  timelineDaysCount = DefaultSettings.allowedDaysCountValues[index];
                  for (int i = 0; i < _selectedTimelineDaysCount.length; i++) {
                    _selectedTimelineDaysCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedTimelineDaysCount,
              children: DefaultSettings.allowedDaysCountValues
                  .map((e) => Text(e.toString()))
                  .toList(),
            ),
          ],
        ),

        // Top Artists
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_top_artists_days_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  topArtistsDaysCount = DefaultSettings.allowedDaysCountValues[index];
                  for (int i = 0; i < _selectedTopArtistsDaysCount.length; i++) {
                    _selectedTopArtistsDaysCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedTopArtistsDaysCount,
              children: DefaultSettings.allowedDaysCountValues
                  .map((e) => Text(e.toString()))
                  .toList(),
            ),
          ],
        ),

        // Discoveries
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_discoveries_days_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  discoveriesDaysCount = DefaultSettings.allowedDaysCountValues[index];
                  for (int i = 0; i < _selectedDiscoveriesDaysCount.length; i++) {
                    _selectedDiscoveriesDaysCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedDiscoveriesDaysCount,
              children: DefaultSettings.allowedDaysCountValues
                  .map((e) => Text(e.toString()))
                  .toList(),
            ),
          ],
        ),

        // Distribution by day/hour + heatmap
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_distribution_days_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  distributionDaysCount = DefaultSettings.allowedDaysCountValues[index];
                  for (int i = 0; i < _selectedDistributionDaysCount.length; i++) {
                    _selectedDistributionDaysCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedDistributionDaysCount,
              children: DefaultSettings.allowedDaysCountValues
                  .map((e) => Text(e.toString()))
                  .toList(),
            ),
          ],
        ),

        SizedBox(height: 8),
        AppTitle2(text: tr('settings_title_counts')),

        // New artists count
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_new_artists_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  newArtistsCount = DefaultSettings.allowedCountValues[index];
                  for (int i = 0; i < _selectedNewArtistsCount.length; i++) {
                    _selectedNewArtistsCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedNewArtistsCount,
              children:
                  DefaultSettings.allowedCountValues.map((e) => Text(e.toString())).toList(),
            ),
          ],
        ),

        // New tracks count
        Row(
          mainAxisAlignment: MainAxisAlignment.end,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text('settings_label_new_tracks_count').tr(),
            ToggleButtons(
              onPressed: (int index) {
                setState(() {
                  newTracksCount = DefaultSettings.allowedCountValues[index];
                  for (int i = 0; i < _selectedNewTracksCount.length; i++) {
                    _selectedNewTracksCount[i] = i == index;
                  }
                });
                saveSettings();
              },
              borderRadius: const BorderRadius.all(Radius.circular(8)),
              constraints: const BoxConstraints(minHeight: 30.0, minWidth: 30.0),
              isSelected: _selectedNewTracksCount,
              children:
                  DefaultSettings.allowedCountValues.map((e) => Text(e.toString())).toList(),
            ),
          ],
        ),

        // Save
        // ElevatedButton(
        //   child: Row(
        //     mainAxisAlignment: MainAxisAlignment.center,
        //     crossAxisAlignment: CrossAxisAlignment.center,
        //     children: [
        //       Icon(UniconsLine.save),
        //       SizedBox(width: 8),
        //       Text('settings_button_save').tr(),
        //     ],
        //   ),
        //   onPressed: () => saveSettings(),
        // ),
      ],
    );
  }
}
